/*
 * This file is part of Jelie,
 * (c) 2002 Julien Pilet <julien.pilet@epfl.ch> and
 * Stephane Magnenat <stephane.magnenat@epfl.ch>
 *
 * Jelie is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * Jelie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*! \file
 * This is the factory that creates the jtag controller 
 * who are, in fact, the physical jtag interfaces
 *
 * \author Julien Pilet <julien.pilet@epfl.ch>
 * \author Stephane Magnenat <stephane.magnenat@epfl.ch>
 */

#ifndef __JTAG_CTRL_FACTORY
#define __JTAG_CTRL_FACTORY

#include "jtag.h"
#include <vector>

//! The factory have an instance of all available jtag controls
class JTAGControlFactory
{
public:
	//! Create the factory, register all available controls
	JTAGControlFactory(void);

	//! Destroy the factory, delete all attached controls
	~JTAGControlFactory(void);
	
	//! Get a control by a given name
	JTAGControl *getPrefered(const char *name, int argc, char *argv[]) const;

	//! Get a control that works. If NULL is return, no control is available
	JTAGControl *getAvailable(int argc, char *argv[]) const;
	
	//! Dump the list of all available drivers
	void dumpList(void) const;
private:
	std::vector<JTAGControl *> controls;	
};

#endif
